#include "stdafx.h"
#include "CvHMM.h"
#include <opencv2\opencv.hpp>
#include <opencv\cv.h>
#include <Windows.h>
#include <opencv2\highgui\highgui.hpp>




#include <opencv2/core/core.hpp>
#include "CvHMM.h"


#include <iostream>
#include <time.h>

//Debug Variable for puju's classes
int inputTest = 0;

int lowerH = 65;
int lowerS = 100;
int lowerV = 100;

int upperH = 80;
int upperS = 255;
int upperV = 255;

int dilIter = 10;
int erIter = 3;


int frameNum;
using namespace cv;


class marker{

public:
	int id;
	String color;
	int x;
	int y;
	int num;
	int xBlock;
	int yBlock;
	marker* next;
	int fNum = frameNum;

	marker()
	{
		x = 0;
		y = 0;
		xBlock = 0;
		yBlock = 0;
		id = 101;
		color = "POP";
		next = NULL;
	}
	marker(int i, String str)
	{
		x = 0;
		y = 0;
		xBlock = 0;
		yBlock = 0;
		id = i;
		color = str;
		next = NULL;
	}

	marker(int i, String str, int xpos, int ypos)
	{
		x = xpos;
		y = ypos;
		xBlock = 0;
		yBlock = 0;
		id = i;
		color = str;
		next = NULL;
	}
};

void setwindowSettings(){

	cvNamedWindow("Selector", WINDOW_NORMAL);

	cvCreateTrackbar("LowerH", "Selector", &lowerH, 180, NULL);
	cvCreateTrackbar("UpperH", "Selector", &upperH, 180, NULL);

	cvCreateTrackbar("LowerS", "Selector", &lowerS, 256, NULL);
	cvCreateTrackbar("UpperS", "Selector", &upperS, 256, NULL);

	cvCreateTrackbar("LowerV", "Selector", &lowerV, 256, NULL);
	cvCreateTrackbar("UpperV", "Selector", &upperV, 256, NULL);

	cvCreateTrackbar("Dilator", "Selector", &dilIter, 15, NULL);
	cvCreateTrackbar("Eroder", "Selector", &erIter, 15, NULL);

}


/*
Function to obtain the threshold Image
*/

Mat GetThresholdedImage(Mat img, Scalar upperThresh, Scalar lowerThresh)
{



	Mat imgHSV;

	// Convert the image into an HSV image
	cvtColor(img, imgHSV, COLOR_BGR2HSV);
	GaussianBlur(imgHSV, imgHSV, Size(3, 3), 1.5);
	// Define a threshold image
	Mat imgThresh;

	Mat eKernel = getStructuringElement(MORPH_RECT, Size(3, 3));
	Mat dKernel = getStructuringElement(MORPH_RECT, Size(3, 3));

	/*
	CvScalar upperThresh = cvScalar(upperH, upperS, upperV);
	CvScalar lowerThresh = cvScalar(lowerH, lowerS, lowerV);

	// Skin Test
	CvScalar upperThresh = cvScalar(20, 150, 255);
	CvScalar lowerThresh = cvScalar(0, 30, 50);
	*/


	inRange(imgHSV, lowerThresh, upperThresh, imgThresh);

	for (int i = 0; i < erIter; i++)
	{
		erode(imgThresh, imgThresh, eKernel);
	}

	for (int i = 0; i < dilIter; i++)
	{
		dilate(imgThresh, imgThresh, dKernel);
	}

	imshow("Thresholded Image", imgThresh);
	return imgThresh;
}

string intToString(int number){

	std::stringstream ss;
	ss << number;
	return ss.str();
}

void drawObject(marker *obj, Mat &frame){

	while (obj->next != 0)
	{
		obj = obj->next;
		int x = obj->x;
		int y = obj->y;

		//flip(frame, frame, 1);

		double fWidth = frame.cols;
		double fHeight = frame.rows;

		Scalar drawColor;
		const char* color = obj->color.c_str();
		const char* blue = "blue";
		const char* red = "red";
		const char* green = "green";

		if (!strcmp(color, blue))
		{
			//std::cout << obj->color;
			drawColor = Scalar(255, 0, 0);
		}
		else if (!strcmp(color, green))
		{
			drawColor = Scalar(0, 255, 0);
		}
		else if (!strcmp(color, red))
		{
			drawColor = Scalar(0, 0, 255);

		}

		circle(frame, Point(x, y), 5, drawColor, 2);
		if (y - 10 > 0)
			line(frame, Point(x, y), Point(x, y - 10), drawColor, 2);
		else line(frame, Point(x, y), Point(x, 0), drawColor, 2);
		if (y + 10<fHeight)
			line(frame, Point(x, y), Point(x, y + 10), drawColor, 2);
		else line(frame, Point(x, y), Point(x, fHeight), drawColor, 2);
		if (x - 10 > 0)
			line(frame, Point(x, y), Point(x - 10, y), drawColor, 2);
		else line(frame, Point(x, y), Point(0, y), drawColor, 2);
		if (x + 10 < fWidth)
			line(frame, Point(x, y), Point(x + 10, y), drawColor, 2);
		else line(frame, Point(x, y), Point(fWidth, y), drawColor, 2);
		//putText(frame, intToString(x) + "," + intToString(y), Point(x, y + 30), 1, 1, drawColor, 2);
	}

}

Mat track(Mat &curFrame, Mat &imgThresh, Scalar upperThresh, Scalar lowerThresh, marker *obj, String color){

	Mat curThresh = GetThresholdedImage(curFrame, upperThresh, lowerThresh);
	Mat out;
	curThresh.copyTo(out);

	vector< vector<Point> > contours;
	vector<Vec4i> hierarchy;
	findContours(curThresh, contours, hierarchy, CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE);

	int count = 0;
	while (obj->next != 0)
	{
		obj = obj->next;
		count++;
	}


	int numObjects = 0;
	double refArea = 0;
	bool objectFound = false;
	if (hierarchy.size() > 0) {
		numObjects = hierarchy.size();
		for (int index = 0; index >= 0; index = hierarchy[index][0])
		{
			Moments moment = moments((cv::Mat)contours[index]);
			double area = moment.m00;
			if (area > (4 * 4))
			{
				marker *cur = new marker(++count, color, moment.m10 / area, moment.m01 / area);
				obj->next = cur;
				obj = obj->next;
				objectFound = true;
			}

		}

	}

	return out;
}

Mat get_xy_data(int *p, int *q, int sizeC){

	if (sizeC == 0)
		return Mat(1, 1, CV_32FC1);
	Mat M(sizeC, 2, CV_32FC1, Scalar(0));

	for (int i = 0; i < sizeC; i++)
		M.at<float>(i, 0) = p[i];
	for (int i = 0; i < sizeC; i++)
		M.at<float>(i, 1) = q[i];

	return M;
}

Mat get_point_centroids(int *p, int *q, int sizeC){


	if (sizeC == 0)
		return Mat(1, 1, CV_32S);

	Mat A = get_xy_data(p, q, sizeC);
	//std::cout << A;
	int k = ceil(sizeC / 2);
	//int k = 2;
	int clusterCount = (k == 0) ? 1 : k;
	Mat labels;
	int attempts = 5;
	Mat centers;
	try
	{
		kmeans(A, clusterCount, labels, TermCriteria(CV_TERMCRIT_ITER | CV_TERMCRIT_EPS, 10000, 0.0001), attempts, KMEANS_PP_CENTERS, centers);
	}
	catch (int)
	{
		centers = Mat(1, 1, CV_32S);
	}
	Mat temp = Mat(centers.rows, centers.cols, CV_32S);
	for (int i = 0; i < centers.rows; i++){
		for (int j = 0; j < centers.cols; j++){
			temp.at<int>(i, j) = ceil(centers.at<float>(i, j));
		}
	}
	return temp;
}


boolean isClose(marker obj1, marker obj2){



	int dist = sqrt(pow((obj1.x) - (obj2.x), 2) + pow((obj1.y) - (obj2.y), 2));
	if (obj1.x == -1 || obj2.x == -1)
		return false;

	return dist > 35 ? false : true;

}


String detectGesture(marker* red, marker* green, marker* blue){
	marker* curRed = red;
	int minX = 0;
	int maxX = 640;
	int minY = 0;
	int maxY = 480;
	/*while (curRed->next != 0)
	{
	if (curRed->x > maxX)
	{
	maxX = curRed->x;
	}
	else if (curRed->x < minX)
	{
	minX = curRed->x;
	}
	else if (curRed->y > maxY)
	{
	maxY = curRed->y;
	}
	else if (curRed->y < minY)
	{
	minY = curRed->y;
	}

	curRed = curRed->next;
	}*/
	//std::cout << "\nmax x " << maxX;
	//std::cout << "\nmin x " << minX;
	int xDiff = maxX - minX;
	int yDiff = maxY - minY;
	xDiff = xDiff / 12;
	yDiff = yDiff / 12;
	xDiff = xDiff != 0 ? xDiff : 1;
	yDiff = yDiff != 0 ? yDiff : 1;
	curRed = red;
	while (curRed->next != 0)
	{

		curRed->xBlock = (curRed->x - minX) / xDiff;
		curRed->yBlock = (curRed->y - minY) / yDiff;

		//std::cout << "\ndiff " << xDiff;

		curRed = curRed->next;
	}

	FILE* file = fopen("gestureData1.txt", "a+");
	fprintf(file, "\n***************\n");
	int prev = -1;
	curRed = red;
	std::string strX = "";
	std::string strY = "";

	char buff[2];
	while (curRed->next != 0)
	{
		if (prev != curRed->xBlock)
		{
			prev = curRed->xBlock;
			strX.append(_itoa(prev, buff, 10));

			fprintf(file, " %i, ", curRed->xBlock);
		}
		//fprintf(file, " %i, ", curRed->yBlock);
		//std::cout << "\nCurrent Pos " << curRed->xBlock;// << ", " << curRed->yBlock;
		curRed = curRed->next;
	}
	fprintf(file, "\n***************\n");
	curRed = red;
	prev = -1;
	while (curRed->next != 0)
	{
		if (prev != curRed->yBlock)
		{
			prev = curRed->yBlock;
			strY.append(_itoa(prev, buff, 10));

			//fprintf(file, " %i, ", curRed->yBlock);
		}
		//fprintf(file, " %i, ", curRed->yBlock);
		//std::cout << "\nCurrent Pos " << curRed->xBlock;// << ", " << curRed->yBlock;
		curRed = curRed->next;
	}
	//std::cout << strY;
	/*
	if (strY.find("567") != std::string::npos && strY.find("765") != std::string::npos)
	{
		if (strX.find("567") != std::string::npos && strX.find("765") != std::string::npos){
			return "Circle";
		}
	}
	else if (strY.find("555") != std::string::npos && strY.find("567") != std::string::npos && strY.find("777") != std::string::npos)
	{
		if (strX.find("765") != std::string::npos && strX.find("567") != std::string::npos && strX.find("765") != std::string::npos){
			return "Z";
		}
	}
	else if (strX.find("3456789") != std::string::npos)
	{
		//std::cout << "Left"; 

		return "Left";
	}
	else if (strX.find("8765432") != std::string::npos)
	{
		//std::cout << "Right";
		return "Right";
	}
	else if (strY.find("3456789") != std::string::npos)
	{
		//std::cout << "Left"; 

		return "Down";
	}
	else if (strY.find("9876543") != std::string::npos)
	{
		//std::cout << "Left"; 

		return "Up";
	}
	*/

	//std::cout << "Left"; 



	curRed = red;

	//fprintf(file, "\n***************\n");
	//while (curRed->next != 0)
	//{
	//	fprintf(file, " %i, ", curRed->yBlock);
	//	//fprintf(file, " %i, ", curRed->yBlock);
	//	//std::cout << "\nCurrent Pos " << curRed->xBlock;// << ", " << curRed->yBlock;
	//	curRed = curRed->next;
	//}
	fclose(file);
	return "";
}

int main()
{
	// Capture Live Feed from camera source
	VideoCapture camSrc;
	camSrc.open(0);
	Mat feed;
	camSrc.read(feed);

	frameNum = 0;
	//setwindowSettings();


	// Opening up windows for debugging
	cvNamedWindow("Actual Feed");
	cvNamedWindow("Thresholded Image");

	/*
	cvNamedWindow("Thresholded Image");
	cvNamedWindow("Canvas");
	cv::moveWindow("Actual Feed", 10, 500);
	cv::moveWindow("Thresholded Image", 650, 500);
	cv::moveWindow("Canvas", 1300, 500);
	*/

	int count = 0;

	// Tracking Marker image
	Mat imgMarker;

	Mat imgThresh;

	Scalar lowerThresh;
	Scalar upperThresh;


	marker red(0, "red");
	marker green(0, "green");
	marker blue(0, "blue");


	int refresh = 0;
	int removeNum = -1;
	int overlayFrame = 0;
	String foundGesture;
	int gestRefresh = 0;


	boolean dclick = false;
	boolean inAction = false;
	cvNamedWindow("Canvas");
	cv::moveWindow("Canvas", 1300, 500);
	IplImage* canvas = cvCreateImage(cvSize(600, 600), 8, 3);

	//Application Loop
	while (true)
	{

		
		int data[5][10] = 		{
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 },
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 },
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 },
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 },
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 }
		};

		int test[5][10] = {
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 },
			{ 1, 2, 3, 3, 2, 1, 2, 3, 2, 1 },
			{ 1, 1, 3, 3, 1, 1, 1, 1, 1, 1 },
			{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
			{ 1, 1, 3, 3, 1, 1, 1, 2, 2, 2 },
		};

		Mat seq = Mat(5, 10, CV_32FC1, &data);
		Mat testseq = Mat(5, 10, CV_32FC1, &test);
		CvHMM hmm;
		double TRGUESSdata[144];
		std::fill_n(TRGUESSdata, 144, 1.0 / 5.0);
		cv::Mat TRGUESS = cv::Mat(12,12 , CV_64F, TRGUESSdata).clone();
		double EMITGUESSdata[144];
		std::fill_n(EMITGUESSdata, 144, 1.0 / 12.0);
		cv::Mat EMITGUESS = cv::Mat(12, 12, CV_64F, EMITGUESSdata).clone();
		double INITGUESSdata[] = { 1, 2, 3, 3, 2, 1, 2, 3, 2, 1, 1 ,2 };
		cv::Mat INITGUESS = cv::Mat(1, 12, CV_64F, INITGUESSdata).clone();

		hmm.printModel(TRGUESS, EMITGUESS, INITGUESS);
		hmm.train(seq, 10, TRGUESS, EMITGUESS, INITGUESS);
		hmm.printModel(TRGUESS, EMITGUESS, INITGUESS);

		cv::Mat estates;
		std::cout << "\n";
		for (int i = 0; i<seq.rows; i++)
		{
			std::cout << i << ": ";
			hmm.viterbi(seq.row(i), TRGUESS, EMITGUESS, INITGUESS, estates);
			for (int i = 0; i<estates.cols; i++)
				std::cout << estates.at<int>(0, i);
			std::cout << "\n";
		}



		cv::Mat pstates, forward, backward;
		double logpseq;
		std::cout << "\n";
		for (int i = 0; i < testseq.rows; i++)
		{
			hmm.decode(testseq.row(i), TRGUESS, EMITGUESS, INITGUESS, logpseq, pstates, forward, backward);
			std::cout << "logpseq" << i << " " << logpseq << "\n";
		}





		getchar();
		
		int idx = 0;
		// Grab a frame from the live feedh
		Mat curFrame;
		camSrc.read(curFrame);



		// Get the X,Y values from last frame (If new set to 0)
		int X = 0;
		int Y = 0;

		if (inputTest == 1)
		{
			setwindowSettings();
			// Track Green
			upperThresh = Scalar(upperH, upperS, upperV);
			lowerThresh = Scalar(lowerH, lowerS, lowerV);

			Mat gnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &green, "Green");
			imgMarker = gnum;
		}
		else
		{

			// Track Green
			upperThresh = Scalar(80, 255, 255);
			lowerThresh = Scalar(65, 100, 100);


			Mat gnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &green, "green");


			// Track Blue
			upperThresh = Scalar(124, 255, 255);
			lowerThresh = Scalar(68, 153, 101);

			Mat bnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &blue, "blue");

			// Track Red
			upperThresh = Scalar(180, 255, 255);
			lowerThresh = Scalar(123, 109, 130);


			upperThresh = Scalar(124, 255, 255);
			lowerThresh = Scalar(68, 153, 101);

			Mat rnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &red, "red");
			imgMarker = rnum + gnum + bnum;
		}


		imshow("Thresholded Image", imgMarker);

		frameNum++;

		if (frameNum > 49)
		{
			refresh = 1;

			if (removeNum == -1)
				removeNum = 0;
		}

		if (removeNum != -1)
			removeNum++;

		if (refresh == 1)
		{
			//std::cout << removeNum;
			while (red.fNum < removeNum && red.next != 0)
				red = *red.next;
			while (green.fNum < removeNum && green.next != 0)
				green = *green.next;
			while (blue.fNum < removeNum && blue.next != 0)
				blue = *blue.next;


		}

		//std::cout << "\nHere are all the X\n";
		if (gestRefresh == 0)
		{
			String gesture = detectGesture(&red, &green, &blue);
			gestRefresh = 30;
			if (gesture.length() != 0)
			{
				std::cout << gesture << "\n";
				while (red.next != 0)
					red = *red.next;

				foundGesture = gesture;
				overlayFrame = 30;
			}
		}
		gestRefresh--;


		if (overlayFrame != -1)
		{
			overlayFrame--;
			flip(curFrame, curFrame, 1);
			putText(curFrame, foundGesture, cvPoint(30, 80),
				FONT_HERSHEY_DUPLEX, 3, cvScalar(0, 0, 250), 3, CV_AA);
			flip(curFrame, curFrame, 1);
		}




		marker curRed = red;
		while (curRed.next != 0)
		{

			curRed = *curRed.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curRed.xBlock = ceil(curRed.x);// (640 / 64));
		curRed.yBlock = ceil(curRed.y); // (480 / 48));
		curRed.x = curRed.fNum == frameNum - 1 ? curRed.x : -1;
		curRed.y = curRed.fNum == frameNum - 1 ? curRed.y : -1;
		//std::cout << "\nCurrent Pos " << curRed.xBlock << ", " << curRed.yBlock;

		marker curGreen = green;
		while (curGreen.next != 0)
		{

			curGreen = *curGreen.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curGreen.x = curGreen.fNum == frameNum - 1 ? curGreen.x : -1;
		curGreen.y = curGreen.fNum == frameNum - 1 ? curGreen.y : -1;


		marker curBlue = blue;
		while (curBlue.next != 0)
		{

			curBlue = *curBlue.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curBlue.x = curBlue.fNum == frameNum - 1 ? curBlue.x : -1;
		curBlue.y = curBlue.fNum == frameNum - 1 ? curBlue.y : -1;

		std::cout << "\nCurrent Pos " << curRed.xBlock/53 << ", " << curRed.yBlock/40;
		/*
		while (cur.next != 0)
		{

		cur = *cur.next;
		std::cout << cur.color << " : "<< cur.id << " (";
		std::cout << cur.x << ", " << cur.y << ") \n";
		}

		cur = green;
		while (cur.next != 0)
		{

		cur = *cur.next;
		std::cout << cur.color << " : " << cur.id << " (";
		std::cout << cur.x << ", " << cur.y << ") \n";
		}

		cur = blue;
		while (cur.next != 0)
		{

		cur = *cur.next;
		std::cout << cur.color << " : " << cur.id << " (";
		std::cout << cur.x << ", " << cur.y << ") \n";
		}

		*/

		// Add the marker to the canvas
		drawObject(&red, curFrame);
		drawObject(&blue, curFrame);
		drawObject(&green, curFrame);
		flip(curFrame, curFrame, 1);
		imshow("Actual Feed", curFrame);



		// **********  Move the mouse on screen  **********

		// Get the frame size
		double fWidth = curFrame.cols;
		double fHeight = curFrame.rows;

		static int Xi = 0;
		static int Yi = 0;

		static int cursorX = 0;
		static int cursorY = 0;

		int prevX = Xi;
		int prevY = Yi;

		// Update the X,Y values
		Xi = curRed.x;
		Yi = curRed.y;

		// Debug
		//printf("position (%d,%d) %d\n", Xi, Yi, count);

		cursorX = abs(cursorX - Xi) > 2 ? fWidth - Xi : fWidth - cursorX;
		cursorY = abs(Yi - cursorY) > 2 ? Yi : cursorY;

		// Scale to the windows  resolution
		double fx = cursorX*(65535.0f / fWidth);
		double fy = cursorY*(65535.0f / fHeight);


		// Send x,y input to the windows API
		/*INPUT  Input = { 0 };
		Input.type = INPUT_MOUSE;
		Input.mi.dwFlags = MOUSEEVENTF_MOVE | MOUSEEVENTF_ABSOLUTE;
		Input.mi.dx = fx;
		Input.mi.dy = fy;
		::SendInput(1, &Input, sizeof(INPUT));*/


		/*
		if (inAction && abs(Y - prevY) < 6 && abs(X - prevX) < 6)
		{
		count++;
		if (count>30 && !dclick)
		{
		Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		printf("CLICK");
		dclick = true;
		}
		if (count>60 && dclick)
		{
		Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		printf("DOUBLE CLICK");

		dclick = false;
		count = 0;
		}

		}
		else
		count = 0;
		*/


		//boolean drag = false;
		//if (isClose(curRed, curBlue) && drag == false)
		//{
		//	std::cout << isClose(curRed, curBlue);
		//	Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP;
		//	::SendInput(1, &Input, sizeof(INPUT));
		//	drag = true; 

		//}

		/*
		else if (drag== true)
		{
		Input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		}
		*/

		if (prevX > 0 && prevY > 0 && Xi > 0 && Yi > 0 && refresh == 1 && curBlue.x != -1)
		{
			cvLine(canvas, cvPoint(fWidth - Xi, Yi), cvPoint(fWidth - prevX, prevY), cvScalar(65, 100, 100), 2);
		}
		cvShowImage("Canvas", canvas);





		//if (refresh == 1 && frameNum % 10 == 0)
		//{
		//	int* count = new int[1];
		//	*count = 0;
		//	std::cout << "count:  " << *count << "\n";
		//	marker cur = red;
		//	while (cur.next != 0)
		//	{
		//		cur = *cur.next;
		//		*(count) = *(count)+1;
		//	}
		//	std::cout << "count:  " << *count << "\n";
		//	int* x = new int[*count];
		//	int* y = new int[*count];
		//	int index = 0;
		//	cur = red;
		//	while (cur.next != 0)
		//	{
		//		cur = *cur.next;
		//		x[index] = cur.x;
		//		y[index] = cur.y;
		//		index++;
		//	}

		//	std::cout << "count:  " << *count << "\n";
		//	std::cout << *x;
		//	Mat centroids = get_point_centroids(x, y, *count);

		//	std::cout << centroids;

		//}


		// Break out of loop if a key is pressed
		int keyp = cvWaitKey(1);
		if (keyp != -1)
		{
			break;
		}

	}

	getchar();
	return 0;
}
