#include "stdafx.h"
#include "CvHMM.h"
#include <opencv2\opencv.hpp>
#include <opencv\cv.h>
#include <Windows.h>
#include <opencv2\highgui\highgui.hpp>
#include <opencv2/core/core.hpp>
#include "CvHMM.h"
#include <iostream>
#include <time.h>
#include <fstream>

//Debug Variable for puju's classes
int inputTest = 0;

int lowerH = 65;
int lowerS = 100;
int lowerV = 100;

int upperH = 80;
int upperS = 255;
int upperV = 255;

int dilIter = 10;
int erIter = 3;


int frameNum;
using namespace cv;


class marker{

public:
	int id;
	String color;
	int x;
	int y;
	int num;
	int xBlock;
	int yBlock;
	marker* next;
	int fNum = frameNum;

	marker()
	{
		x = 0;
		y = 0;
		xBlock = 0;
		yBlock = 0;
		id = 101;
		color = "POP";
		next = NULL;
	}
	marker(int i, String str)
	{
		x = 0;
		y = 0;
		xBlock = 0;
		yBlock = 0;
		id = i;
		color = str;
		next = NULL;
	}

	marker(int i, String str, int xpos, int ypos)
	{
		x = xpos;
		y = ypos;
		xBlock = 0;
		yBlock = 0;
		id = i;
		color = str;
		next = NULL;
	}
};


marker curRed;
marker curGreen;
marker curBlue;
marker curYellow;



void setwindowSettings(){

	cvNamedWindow("Selector", WINDOW_NORMAL);

	cvCreateTrackbar("LowerH", "Selector", &lowerH, 180, NULL);
	cvCreateTrackbar("UpperH", "Selector", &upperH, 180, NULL);

	cvCreateTrackbar("LowerS", "Selector", &lowerS, 256, NULL);
	cvCreateTrackbar("UpperS", "Selector", &upperS, 256, NULL);

	cvCreateTrackbar("LowerV", "Selector", &lowerV, 256, NULL);
	cvCreateTrackbar("UpperV", "Selector", &upperV, 256, NULL);

	cvCreateTrackbar("Dilator", "Selector", &dilIter, 15, NULL);
	cvCreateTrackbar("Eroder", "Selector", &erIter, 15, NULL);

}

/*
Function to obtain the threshold Image
*/
Mat GetThresholdedImage(Mat img, Scalar upperThresh, Scalar lowerThresh)
{



	Mat imgHSV;

	// Convert the image into an HSV image
	cvtColor(img, imgHSV, COLOR_BGR2HSV);
	GaussianBlur(imgHSV, imgHSV, Size(3, 3), 1.5);
	// Define a threshold image
	Mat imgThresh;

	Mat eKernel = getStructuringElement(MORPH_RECT, Size(3, 3));
	Mat dKernel = getStructuringElement(MORPH_RECT, Size(3, 3));

	/*
	CvScalar upperThresh = cvScalar(upperH, upperS, upperV);
	CvScalar lowerThresh = cvScalar(lowerH, lowerS, lowerV);

	// Skin Test
	CvScalar upperThresh = cvScalar(20, 150, 255);
	CvScalar lowerThresh = cvScalar(0, 30, 50);
	*/


	inRange(imgHSV, lowerThresh, upperThresh, imgThresh);

	for (int i = 0; i < erIter; i++)
	{
		erode(imgThresh, imgThresh, eKernel);
	}

	for (int i = 0; i < dilIter; i++)
	{
		dilate(imgThresh, imgThresh, dKernel);
	}

	imshow("Thresholded Image", imgThresh);
	return imgThresh;
}

string intToString(int number){

	std::stringstream ss;
	ss << number;
	return ss.str();
}

void drawObject(marker *obj, Mat &frame){

	while (obj->next != 0)
	{
		obj = obj->next;
		int x = obj->x;
		int y = obj->y;

		//flip(frame, frame, 1);

		double fWidth = frame.cols;
		double fHeight = frame.rows;

		Scalar drawColor;
		const char* color = obj->color.c_str();
		const char* blue = "blue";
		const char* red = "red";
		const char* green = "green";
		const char* yellow = "yellow";

		if (!strcmp(color, blue))
		{
			//std::cout << obj->color;
			drawColor = Scalar(255, 0, 0);
		}
		else if (!strcmp(color, green))
		{
			drawColor = Scalar(0, 255, 0);
		}
		else if (!strcmp(color, red))
		{
			drawColor = Scalar(0, 0, 255);

		}
		else if (!strcmp(color, yellow))
		{
			drawColor = Scalar(0, 255, 255);

		}

		circle(frame, Point(x, y), 5, drawColor, 2);
		if (y - 10 > 0)
			line(frame, Point(x, y), Point(x, y - 10), drawColor, 2);
		else line(frame, Point(x, y), Point(x, 0), drawColor, 2);
		if (y + 10<fHeight)
			line(frame, Point(x, y), Point(x, y + 10), drawColor, 2);
		else line(frame, Point(x, y), Point(x, fHeight), drawColor, 2);
		if (x - 10 > 0)
			line(frame, Point(x, y), Point(x - 10, y), drawColor, 2);
		else line(frame, Point(x, y), Point(0, y), drawColor, 2);
		if (x + 10 < fWidth)
			line(frame, Point(x, y), Point(x + 10, y), drawColor, 2);
		else line(frame, Point(x, y), Point(fWidth, y), drawColor, 2);
		//putText(frame, intToString(x) + "," + intToString(y), Point(x, y + 30), 1, 1, drawColor, 2);
	}
}

Mat track(Mat &curFrame, Mat &imgThresh, Scalar upperThresh, Scalar lowerThresh, marker *obj, String color){

	Mat curThresh = GetThresholdedImage(curFrame, upperThresh, lowerThresh);
	Mat out;
	curThresh.copyTo(out);

	vector< vector<Point> > contours;
	vector<Vec4i> hierarchy;
	findContours(curThresh, contours, hierarchy, CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE);

	int count = 0;
	while (obj->next != 0)
	{
		obj = obj->next;
		count++;
	}


	int numObjects = 0;
	double refArea = 0;
	bool objectFound = false;
	if (hierarchy.size() > 0) {
		numObjects = hierarchy.size();
		for (int index = 0; index >= 0; index = hierarchy[index][0])
		{
			Moments moment = moments((cv::Mat)contours[index]);
			double area = moment.m00;
			if (area > (4 * 4))
			{
				marker *cur = new marker(++count, color, moment.m10 / area, moment.m01 / area);
				obj->next = cur;
				obj = obj->next;
				objectFound = true;
			}

		}

	}

	return out;
}

boolean isClose(marker obj1, marker obj2){

	int dist = sqrt(pow((obj1.x) - (obj2.x), 2) + pow((obj1.y) - (obj2.y), 2));
	if (obj1.x == -1 || obj2.x == -1)
		return false;

	return dist > 40 ? false : true;

}


boolean isFar(marker obj1, marker obj2){



	int dist = sqrt(pow((obj1.x) - (obj2.x), 2) + pow((obj1.y) - (obj2.y), 2));
	//std::cout << "\n" << dist;
	if (obj1.x == -1 || obj2.x == -1)
		return false;

	return dist > 50 ? false : true;

}


// The following will return 404 when change in direction, +1 for increasing (THAT IS THE LEFT DIRECTION), -1 for decreasing (THAT IS THE RIGHT DIRECTION)
// 0 for the same
int getDirection(String inputString){

	int equality = 0;
	int larger = 0;
	int smaller = 0;
	int count = 4;
	int values[5] = { 0 };
	char comma = ',';
	for (int i = inputString.length() - 1; i > 0; i--){
		if (count == -1)
			break;
		if (inputString[i] == comma)
			continue;
		if (inputString[i - 1] == comma){
			values[count] = inputString[i] - '0';
		}
		else{
			int temp1 = inputString[i] - '0';
			int temp = inputString[i - 1] - '0';
			values[count] = temp * 10 + temp1;
			i--;
			//values[count] = temp[i - 1] * 10 + temp[i];
		}
		count--;
	}
	/*for (int i = temp.length() - 4; i < temp.length(); i++){
	if (temp[i] - temp[i - 1] > 0)
	larger++;
	else if (temp[i] == temp[i - 1])
	equality++;
	else
	smaller++;
	}
	if (smaller > 0 && larger > 0)
	return 404;
	else if (smaller == 0 && larger == 0)
	return 0;
	else if (smaller > 0)
	return -1;
	else
	return 1;*/
	/*std::cout << "\nvalues are everything\n";
	for (int i = 0; i < 5; i++)
	std::cout << values[i] << " \t";
	return 1;*/


	for (int i = 1; i < 5; i++){
		if (values[i] - values[i - 1] > 0)
			larger++;
		else if (values[i] == values[i - 1])
			equality++;
		else
			smaller++;
	}

	if (smaller > 0 && larger > 0)
		return 404;
	else if (smaller == 0 && larger == 0)
		return 0;
	else if (smaller > 0)
		return -1;
	else
		return 1;

}


String detectCircle(String strX, String strY, int size){
	//int xMin = atoi(strX[0]);
	//int xMax = atoi(strY[0]);
}

String detectGesture(marker* red, int mode, int *temp){
	marker* curRed = red;
	int minX = 0;
	int maxX = 640;
	int minY = 0;
	int maxY = 480;
	/*while (curRed->next != 0)
	{
	if (curRed->x > maxX)
	{
	maxX = curRed->x;
	}
	else if (curRed->x < minX)
	{
	minX = curRed->x;
	}
	else if (curRed->y > maxY)
	{
	maxY = curRed->y;
	}
	else if (curRed->y < minY)
	{
	minY = curRed->y;
	}
	curRed = curRed->next;
	}*/
	//std::cout << "\nmax x " << maxX;
	//std::cout << "\nmin x " << minX;
	int xDiff = maxX - minX;
	int yDiff = maxY - minY;
	xDiff = xDiff / 12;
	yDiff = yDiff / 12;
	xDiff = xDiff != 0 ? xDiff : 1;
	yDiff = yDiff != 0 ? yDiff : 1;
	curRed = red;
	while (curRed->next != 0)
	{
		curRed->xBlock = (curRed->x - minX) / xDiff;
		curRed->yBlock = (curRed->y - minY) / yDiff;
		//std::cout << "\ndiff " << xDiff;
		curRed = curRed->next;
	}

	//FILE* file = fopen("gestureData1.txt", "a+");
	//fprintf(file, "\n***************\n");

	int prev = -1;
	curRed = red;
	std::string strX = "";
	std::string strY = "";
	std::string strdX = "";
	std::string strdY = "";
	char buff[2];
	while (curRed->next != 0)
	{
		//strdX.append(_itoa(prev, buff, 10));
		//strdX.append(",");
		if (prev != curRed->xBlock)
		{
			prev = curRed->xBlock;
			strX.append(_itoa(prev, buff, 10));
			strdX.append(_itoa(prev, buff, 10));
			strdX.append(",");
			//fprintf(file, " %i, ", curRed->xBlock);
		}
		//fprintf(file, " %i, ", curRed->yBlock);
		//std::cout << "\nCurrent Pos " << curRed->xBlock;// << ", " << curRed->yBlock;
		curRed = curRed->next;
	}
	//	fprintf(file, "\n***************\n");
	curRed = red;
	prev = -1;
	while (curRed->next != 0)
	{
		//strdY.append(_itoa(prev, buff, 10));
		//strdY.append(",");
		if (prev != curRed->yBlock)
		{
			prev = curRed->yBlock;
			strY.append(_itoa(prev, buff, 10));
			strdY.append(_itoa(prev, buff, 10));
			strdY.append(",");
			//fprintf(file, " %i, ", curRed->yBlock);
		}
		//fprintf(file, " %i, ", curRed->yBlock);
		//std::cout << "\nCurrent Pos " << curRed->xBlock;// << ", " << curRed->yBlock;
		curRed = curRed->next;
	}
	//std::cout << strY;
	///MERA KARAN ARJUN AA GYA
	//std::cout << "\nhere is the x string \t" << strX;
	//std::cout << "\nhere is the modified x string \t" << strdX;
	//std::cout << "\nhere is the y string \t" << strY;
	//std::cout << "\nhere is the modified y string \t" << strdY;
	//YOU CAN USE THE X AND THE Y VARIABLEES TO KNOW THE DIRECTION. TO KNOW WHAT THEY RETURN, SEE THE COMMENTS ABOVE THE FUNCTION 
	int x = getDirection(strdX);
	//std::cout << "\n direction\t"<<x;
	int y = getDirection(strdY);
	//std::cout << "\n direction\t" << x;
	/*getchar();
	x = getDirection("11,10,9,8,7");
	//std::cout << "\n direction\t" << x;
	getchar();
	x = getDirection("11,10,11,8,7");
	//std::cout << "\n direction\t" << x;
	getchar();
	int y = getDirection(strdY);
	*/
	//std::cout << "see the values here" << strX<<"\n";
	//std::cout << "\nvalue of the last element of the string" << strX[strX.length()-1];
	*temp = -1;
	if (strdX.length() > 3)
	{
		if (strdX[strdX.length() - 3] == ',')
			*temp = strdX[strdX.length() - 2] - '0';
		else{
			*temp = strdX[strdX.length() - 2] - '0';
			*temp = *temp + 10;
		}
	}


	if (mode == 1){
		if (y == -1)
			return "Up";
		if (y == 1)
			return "Down";
		else
			return "";
	}
	else{
		if (strY.find("567") != std::string::npos && strY.find("765") != std::string::npos)
		{
			if (strX.find("567") != std::string::npos && strX.find("765") != std::string::npos){
				return "Circle";
			}
		}
		else if (strY.find("555") != std::string::npos && strY.find("567") != std::string::npos && strY.find("777") != std::string::npos)
		{
			if (strX.find("765") != std::string::npos && strX.find("567") != std::string::npos && strX.find("765") != std::string::npos){
				return "Z";
			}
		}
		else if (strX.find("3456789") != std::string::npos)
		{
			//std::cout << "Left";
			return "SwipeLeft";
		}
		else if (strX.find("8765432") != std::string::npos)
		{
			//std::cout << "Right";
			return "SwipeRight";
		}
		else if (strY.find("3456789") != std::string::npos)
		{
			//std::cout << "Left";
			return "SwipeDown";
		}
		else if (strY.find("9876543") != std::string::npos)
		{
			//std::cout << "Left";
			return "SwipeUp";
		}
		//std::cout << "Left"; 
		curRed = red;
		//fprintf(file, "\n***************\n");
		//while (curRed->next != 0)
		//{
		//	fprintf(file, " %i, ", curRed->yBlock);
		//	//fprintf(file, " %i, ", curRed->yBlock);
		//	//std::cout << "\nCurrent Pos " << curRed->xBlock;// << ", " << curRed->yBlock;
		//	curRed = curRed->next;
		//}
		//fclose(file);
		return "";
	}
}


//here the order of the markers are important. For instance,
/*
left and right with respect to the user looking at his/her hand
RGBB (CLOCKWISE)
red = index finger left hand
green = index finger for the right hand
blue = pinky of the right hand
black = pinky of the left hand
*/

int counter = 0;
int flag = 0;


boolean pictureReady(){
	//std::cout << curGreen.x;
	if (curRed.x < 4 && curBlue.x > curRed.x && curGreen.x > 8 && curYellow.x < curGreen.x)
		return true;
	else return false;
}

String detectMultipleGesture(marker* mred, marker* mgreen, marker* mblue, marker* myellow, char* output){
	//std::ifstream myfile("C:\Users\pujuncool\Desktop\WindowsApplication2\WindowsApplication2\WindowsApplication2\bin\Debug\IO\IO.txt");
	//std::ifstream myfile("hello.txt");
	//string line;
	//std::getline(myfile, line);
	/*std::fstream fs;
	fs.open("test.txt", std::fstream::in);
	std::cout << line;*/
	//FILE* file = fopen("hello.txt", "r"); /* should check the result */
	//char line[256];

	//while (fgets(line, sizeof(line), file)) {
	//	/* note that fgets don't strip the terminating \n, checking its
	//	presence would allow to handle lines longer that sizeof(line) */
	//	printf("%s", line);
	//}


	std::ifstream myReadFile;
	myReadFile.open("C:\\Users\\Nakul\\Documents\\Visual Studio 2013\\Projects\\WindowsApplication2\\IO\\IO.txt");

	if (myReadFile.is_open()) {
		while (!myReadFile.eof()) {


			myReadFile >> output;
			//std::cout << output[3];


		}
	}

	int rx = -1;
	int gx = -1;
	int bx = -1;
	int yx = -1;
	String red = detectGesture(mred, 1, &rx);
	String green = detectGesture(mgreen, 1, &gx);
	String blue = detectGesture(mblue, 1, &bx);
	String yellow = detectGesture(myellow, 1, &yx);
	counter--;
	//std::cout << red;
	//Okay, so we will have all the combos here
	//std::cout <<"see this yooyoyoyoyoyoyoy"<< rx<<"\n";
	if (rx != -1 && bx != -1 && gx != -1 && yx != -1){
		std::cout << " \nRED " << rx;
		std::cout << " \nGREEN " << gx;
		std::cout << " \nBLUE " << bx;
		std::cout << " \nYELLOW " << yx;


		if (output[3] == 'W'){
			std::cout << "Success";
		}
		else if (rx > 8 && bx < rx && gx < 4 && yx > gx)
			return "Cheese";
	}
	else if (red == "Down"  && isClose(curRed, curBlue) && output[3] == 'P'){
		counter = 50;
		flag = 1;
		return "LeftPinchIN";
	}
	else if (green == "Down"  && isClose(curGreen, curYellow)){
		counter = 50;
		flag = 1;
		return "RightPinchIN";
	}
	else if (output[3] == 'S' || output[3] == 'B'){
		red = detectGesture(mred, 0, &rx);
		return red;
	}

	else
		return "";
}


int main()
{
	// Capture Live Feed from camera source
	VideoCapture camSrc;
	camSrc.open(0);
	Mat feed;
	//camSrc.set(CV_CAP_PROP_EXPOSURE, 3.0);
	camSrc.read(feed);
	frameNum = 0;
	//setwindowSettings();


	// Opening up windows for debugging
	cvNamedWindow("Actual Feed");
	cvNamedWindow("Thresholded Image");

	/*
	cvNamedWindow("Thresholded Image");
	cvNamedWindow("Canvas");
	cv::moveWindow("Actual Feed", 10, 500);
	cv::moveWindow("Thresholded Image", 650, 500);
	cv::moveWindow("Canvas", 1300, 500);
	*/

	int count = 0;

	// Tracking Marker image
	Mat imgMarker;

	Mat imgThresh;

	Scalar lowerThresh;
	Scalar upperThresh;


	marker red(0, "red");
	marker green(0, "green");
	marker blue(0, "blue");
	marker yellow(0, "yellow");


	int refresh = 0;
	int removeNum = -1;
	int overlayFrame = 0;
	String foundGesture;
	int gestRefresh = 0;


	boolean fml = false;
	boolean drag = false;
	boolean dclick = false;
	boolean inAction = false;
	boolean gestureAction = false;

	int clicktimer = 0;

	cvNamedWindow("Canvas");
	cv::moveWindow("Canvas", 1300, 500);
	IplImage* canvas = cvCreateImage(cvSize(600, 600), 8, 3);


	Mat firstFrame;
	camSrc.read(firstFrame);

	char output[100];

	//Application Loop
	while (true)
	{
		//std::cout << camSrc.get(CV_CAP_PROP_EXPOSURE);
		int idx = 0;
		// Grab a frame from the live feedh
		Mat curFrame;
		camSrc.read(curFrame);

		/*
		cv::Mat deltaImg;
		cv::absdiff(curFrame, firstFrame, deltaImg);
		//imshow("delta", deltaImg);
		cv::Mat grayscale;
		cv::cvtColor(deltaImg, grayscale, CV_BGR2GRAY);
		// create a mask that includes all pixel that changed their value
		cv::Mat mask = grayscale>90;
		//imshow("Mask", mask);
		cv::Mat output;
		curFrame.copyTo(output, mask);
		imshow("output", output);
		*/

		// Get the X,Y values from last frame (If new set to 0)
		int X = 0;
		int Y = 0;

		if (inputTest == 1)
		{
			setwindowSettings();
			// Track Green
			upperThresh = Scalar(upperH, upperS, upperV);
			lowerThresh = Scalar(lowerH, lowerS, lowerV);

			Mat gnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &green, "Green");
			imgMarker = gnum;
		}
		else
		{

			// Track Green
			upperThresh = Scalar(84, 230, 91);
			lowerThresh = Scalar(59, 153, 22);


			Mat gnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &green, "green");


			// Track Blue
			upperThresh = Scalar(131, 255, 179);
			lowerThresh = Scalar(95, 167, 125);

			Mat bnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &blue, "blue");

			// Track Red
			upperThresh = Scalar(180, 255, 181);
			lowerThresh = Scalar(130, 139, 62);


			//upperThresh = Scalar(124, 255, 255);
			//lowerThresh = Scalar(68, 153, 101);

			Mat rnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &red, "red");

			upperThresh = Scalar(59, 115, 256);
			lowerThresh = Scalar(27, 46, 171);

			Mat pnum = track(curFrame, imgThresh, upperThresh, lowerThresh, &yellow, "yellow");
			imgMarker = bnum + rnum + gnum + pnum;
		}


		imshow("Thresholded Image", imgMarker);

		frameNum++;

		if (frameNum > 25)
		{
			refresh = 1;

			if (removeNum == -1)
				removeNum = 0;
		}

		if (removeNum != -1)
			removeNum++;

		if (refresh == 1)
		{
			//std::cout << removeNum;
			while (red.fNum < removeNum && red.next != 0)
				red = *red.next;
			while (green.fNum < removeNum && green.next != 0)
				green = *green.next;
			while (blue.fNum < removeNum && blue.next != 0)
				blue = *blue.next;
			while (yellow.fNum < removeNum && yellow.next != 0)
				yellow = *yellow.next;
		}

		//std::cout << "\nHere are all the X\n";
		if (gestRefresh == 0)
		{
			String gesture;

			gesture = detectMultipleGesture(&red, &green, &blue, &yellow, output);
			//gesture = detectGesture(&red);
			gestRefresh = 10;
			if (gesture.length() != 0)
			{
				std::cout << gesture << "\n";
				while (red.next != 0)
					red = *red.next;
				while (blue.next != 0)
					blue = *blue.next;
				while (green.next != 0)
					green = *green.next;
				while (yellow.next != 0)
					yellow = *yellow.next;

				foundGesture = gesture;
				gestureAction = true;
				overlayFrame = 30;
			}
		}
		gestRefresh--;

		if (overlayFrame != -1)
		{
			overlayFrame--;
			flip(curFrame, curFrame, 1);
			putText(curFrame, foundGesture, cvPoint(30, 80),
				FONT_HERSHEY_DUPLEX, 3, cvScalar(0, 0, 250), 3, CV_AA);
			flip(curFrame, curFrame, 1);
		}
		else if (gestureAction)
		{
			if (foundGesture == "LeftPinchIN" && output[3] == 'P')
			{
				std::cout << "Screenshot!";
				char buffer[200];
				sprintf(buffer, "Z:/public_html/ConnecTied101/shots/%i.jpg", frameNum);
				std::cout << imwrite(buffer, curFrame);
				imshow("Photo", curFrame);
				//ShellExecute(NULL, "open", link.c_str(), NULL, NULL, SW_SHOWNORMAL);
				//sprintf(buffer, "start http://images.google.com/searchbyimage?site=search^&image_url=http://www.prism.gatech.edu/~ndureja3/ConnecTied101/shots/%i", frameNum);
				//strcat(buffer, ".jpg");
				//std::cout << buffer;
				//system(buffer);
			}

			if (foundGesture == "SwipeLeft" && output[3] == 'S')
			{
				std::cout << "Screenshot!";
				char buffer[200];
				sprintf(buffer, "Z:/public_html/ConnecTied101/shots/%i.jpg", frameNum);
				std::cout << imwrite(buffer, curFrame);
				//ShellExecute(NULL, "open", link.c_str(), NULL, NULL, SW_SHOWNORMAL);
				sprintf(buffer, "start http://images.google.com/searchbyimage?site=search^&image_url=http://www.prism.gatech.edu/~ndureja3/ConnecTied101/shots/%i", frameNum);
				strcat(buffer, ".jpg");
				std::cout << buffer;
				system(buffer);
			}

			if (foundGesture == "SwipeRight" && output[3] == 'B')
			{
				std::cout << "Barcode Reader!";
				char buffer[200];
				sprintf(buffer, "Z:/public_html/ConnecTied101/shots/%i.jpg", frameNum);
				std::cout << imwrite(buffer, curFrame);
				//ShellExecute(NULL, "open", link.c_str(), NULL, NULL, SW_SHOWNORMAL);
				//sprintf(buffer, "start http://images.google.com/searchbyimage?site=search^&image_url=http://www.prism.gatech.edu/~ndureja3/ConnecTied101/shots/%i", frameNum);
				sprintf(buffer, "start http://zxing.org/w/decode?u=http://www.prism.gatech.edu/~ndureja3/ConnecTied101/shots/%i", frameNum);
				strcat(buffer, ".jpg");
				std::cout << buffer;
				system(buffer);
			}

			gestureAction = false;
		}

		curRed = red;
		while (curRed.next != 0)
		{

			curRed = *curRed.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curRed.xBlock = ceil(curRed.x);// (640 / 64));
		curRed.yBlock = ceil(curRed.y); // (480 / 48));
		curRed.x = curRed.fNum == frameNum - 1 ? curRed.x : -1;
		curRed.y = curRed.fNum == frameNum - 1 ? curRed.y : -1;
		//std::cout << "\nCurrent Pos " << curRed.xBlock << ", " << curRed.yBlock;

		curGreen = green;
		while (curGreen.next != 0)
		{

			curGreen = *curGreen.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curGreen.x = curGreen.fNum == frameNum - 1 ? curGreen.x : -1;
		curGreen.y = curGreen.fNum == frameNum - 1 ? curGreen.y : -1;


		curBlue = blue;
		while (curBlue.next != 0)
		{

			curBlue = *curBlue.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curBlue.x = curBlue.fNum == frameNum - 1 ? curBlue.x : -1;
		curBlue.y = curBlue.fNum == frameNum - 1 ? curBlue.y : -1;


		curYellow = yellow;
		while (curYellow.next != 0)
		{

			curYellow = *curYellow.next;
			//std::cout << cur.color << " : " << cur.id << " (";
			//std::cout << cur.x << ", " << cur.y << ") \n";
		}
		curYellow.x = curYellow.fNum == frameNum - 1 ? curYellow.x : -1;
		curYellow.y = curYellow.fNum == frameNum - 1 ? curYellow.y : -1;


		//std::cout << "\nCurrent Pos " << curRed.xBlock / 53 << ", " << curRed.yBlock / 40;
		/*
		while (cur.next != 0)
		{

		cur = *cur.next;
		std::cout << cur.color << " : "<< cur.id << " (";
		std::cout << cur.x << ", " << cur.y << ") \n";
		}

		cur = green;
		while (cur.next != 0)
		{

		cur = *cur.next;
		std::cout << cur.color << " : " << cur.id << " (";
		std::cout << cur.x << ", " << cur.y << ") \n";
		}

		cur = blue;
		while (cur.next != 0)
		{

		cur = *cur.next;
		std::cout << cur.color << " : " << cur.id << " (";
		std::cout << cur.x << ", " << cur.y << ") \n";
		}

		*/

		// Add the marker to the canvas
		drawObject(&red, curFrame);
		drawObject(&blue, curFrame);
		drawObject(&green, curFrame);
		drawObject(&yellow, curFrame);
		flip(curFrame, curFrame, 1);
		imshow("Actual Feed", curFrame);

		// **********  Move the mouse on screen  **********

		// Get the frame size
		double fWidth = curFrame.cols;
		double fHeight = curFrame.rows;

		static int Xi = 0;
		static int Yi = 0;

		static int cursorX = 0;
		static int cursorY = 0;

		int prevX = Xi;
		int prevY = Yi;

		// Update the X,Y values
		Xi = curRed.x;
		Yi = curRed.y;

		// Debug
		//printf("position (%d,%d) %d\n", Xi, Yi, count);

		cursorX = abs(cursorX - Xi) > 2 ? fWidth - Xi : fWidth - cursorX;
		cursorY = abs(Yi - cursorY) > 2 ? Yi : cursorY;

		// Scale to the windows  resolution
		double fx = cursorX*(65535.0f / (fWidth - 50));
		double fy = cursorY*(65535.0f / (fHeight - 50));


		// Send x,y input to the windows API
		INPUT  Input = { 0 };
		Input.type = INPUT_MOUSE;
		Input.mi.dwFlags = MOUSEEVENTF_MOVE | MOUSEEVENTF_ABSOLUTE;

		if (Xi >1)
		{
			Input.mi.dx = fx;
			Input.mi.dy = fy;
			::SendInput(1, &Input, sizeof(INPUT));
		}


		
		if (isClose(curRed, curBlue) && drag == false && clicktimer==0 && dclick==false)
		{
		std::cout << isClose(curRed, curBlue);
		Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		drag = true;
		dclick = true;
		clicktimer = 35;
		}
		else if (fml == false && drag == true && isClose(curRed, curBlue))
		{
		Input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN;
		::SendInput(1, &Input, sizeof(INPUT));
		fml = true;
		}
		else if(!isClose(curRed, curBlue))
		{
		Input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));
		drag = false;
		fml = false;
		dclick = false;
		clicktimer = 0;
		}
		if (drag == false)
		{
		Input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
		::SendInput(1, &Input, sizeof(INPUT));

		}

		

		if (clicktimer != 0)
		{
			clicktimer--;
		}

		if (prevX > 0 && prevY > 0 && Xi > 0 && Yi > 0 && refresh == 1 && curBlue.x != -1)
		{
			cvLine(canvas, cvPoint(fWidth - Xi, Yi), cvPoint(fWidth - prevX, prevY), cvScalar(65, 100, 100), 2);
		}
		cvShowImage("Canvas", canvas);

		// Break out of loop if a key is pressed
		int keyp = cvWaitKey(1);
		if (keyp != -1)
		{
			break;
		}

	}
	getchar();
	return 0;
}